// This is a simple example
package main

import (
	"strings"

	"go.wit.com/gui"
	"go.wit.com/log"
)

func createUnreleaseBox(box *gui.Node) {
	group := release.box.NewGroup("undo and redo things")

	group.NewButton("find the unreleased", func() {
		me.Disable()
		defer me.Enable()
		for _, repo := range me.allrepos {
			if whitelist(repo.String()) {
				log.Info("skipping whitelist", repo.String())
				continue
			}

			if repo.lookToUnwind() {
				log.Info("found something to unwind:", repo.String())
				if setCurrentRepo(repo, "rewind this", "very sure") {
				}
				return
			}
		}
	})
	release.unreleaseB = group.NewButton("re-release", func() {
		me.Disable()
		if release.releaseVersionB.String() != "un-release version "+release.version.String() {
			log.Warn("something went wrong with the release.version:", release.version.String())
			return
		}

		if strings.HasPrefix(release.version.String(), "v") {
			log.Warn("everything is ok. version starts with v.", release.version.String())
		} else {
			log.Warn("version does not start with v.", release.version.String())
			return
		}

		if release.current.status.CheckDirty() {
			log.Info("sorry, it's still dirty")
			return
		}

		curName := release.current.status.GetCurrentBranchName()
		mName := release.current.status.GetMasterBranchName()
		if curName != mName {
			log.Info("\trepo is not working from main branch", curName, "!=", mName)
			return
		}

		log.Info("\treset to devel", curName, release.version.String(), release.reasonS)

		var all [][]string
		all = append(all, []string{"git", "checkout", "devel"})
		all = append(all, []string{"git", "branch", "-D", mName})
		all = append(all, []string{"git", "branch", mName})
		all = append(all, []string{"git", "checkout", mName})
		all = append(all, []string{"git", "push", "--set-upstream", "--force", "origin", mName})

		all = append(all, []string{"git", "tag", "--delete", release.version.String()})
		all = append(all, []string{"git", "push", "--delete", "origin", release.version.String()})
		all = append(all, []string{"git", "tag", "-m", release.reasonS, release.version.String()})
		all = append(all, []string{"git", "push", "origin", release.version.String()})

		if doAll(release.current, all) {
			log.Info("EVERYTHING OK")
			me.Enable()
		} else {
			log.Info("SOMETHING FAILED")
		}
	})
}

func doAll(r *repo, all [][]string) bool {
	for _, cmd := range all {
		log.Info("doAll() RUNNING: cmd =", cmd)
		err, out := r.status.RunCmd(cmd)
		log.Info("doAll() err =", err)
		log.Info("doAll() out =", out)
		if err != nil {
			return false
		}
	}
	return true
}
